/***************************************************************************
                                 ibtest.c
                             -------------------

Example program which uses gpib c library.  More complex than an example
program needs to be really, but useful for testing library functions.

    copyright            : (C) 2002 by Frank Mori Hess
    email                : fmhess@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <getopt.h>
#include "gpib/ib.h"

void fprint_status( FILE* filep, char *msg  );

char outbuffer[ 1024 ]= ":INP:FILT:FREQ?";
char inbuffer[ 1025 ];

typedef struct
{
	int minor;
} parsed_options_t;

static void parse_options( int argc, char *argv[], parsed_options_t *parsed_opts )
{
	int c, index;

	struct option options[] =
	{
		{ "minor", required_argument, NULL, 'm' },
		{ 0 },
	};

	parsed_opts->minor = 0;

	while(1)
	{
		c = getopt_long(argc, argv, "m:", options, &index);
		if( c == -1 ) break;
		switch( c )
		{
		case 0:
			break;
		case 'm':
			parsed_opts->minor = strtol( optarg, NULL, 0 );
			break;
		default:
			fprintf(stderr, "ibtest: invalid option\n");
			exit(1);
		}
	}
}


/* returns a device descriptor after prompting user for primary address */
int prompt_for_device(int minor)
{
	int ud, pad;
	const int sad = 0;
	const int send_eoi = 1;
	const int eos_mode = 0;
	const int timeout = T1s;
	char input[100];

	pad = 20;

	printf("trying to open pad = %i on /dev/gpib%i ...", pad, minor);
	ud = ibdev(minor, pad, sad, timeout, send_eoi, eos_mode);
	if(ud < 0)
	{
		fprint_status( stderr, "ibdev error\n");
		abort();
	}
	else
	{
		printf("success\n");
	}
	
	return ud;
}


int perform_read(int ud)
{
	char *buffer = inbuffer;
	int is_string;
	int i;
	int buffer_size = 1024 + 1;
	memset (buffer, 0, 1024);
	//printf("trying to read %i bytes from device...\n", 1024);

	ibrd(ud, buffer, buffer_size - 1);
	is_string = 1;
	for(i = 0; i < ThreadIbcntl(); ++i)
	{
		if(isascii(buffer[i]) == 0) 
		{
			is_string = 0;
			break;
		}
	}
	if(is_string)
	{
		//printf("received string: '%s'\n", buffer);
	}else
	{
		//printf("received binary data (hex): ");
		for(i = 0; i < ThreadIbcntl(); ++i)
		{
			printf("%2x ", (unsigned)buffer[i]);
		}
		printf("\n");
	}
	//printf("Number of bytes read: %li\n", ThreadIbcntl());
	if(ThreadIbsta() & ERR)
		return -1;
	return 0;
}

int prompt_for_write(int ud)
{
	//printf("sending string: %s\n", buffer);
	if( ibwrt(ud, outbuffer, strlen(outbuffer)) & ERR )
	{
		return -1;
	}
	return 0;
}


int main(int argc, char **argv)
{
	int dev;
	parsed_options_t parsed_opts;

	parse_options( argc, argv, &parsed_opts );

	dev = prompt_for_device(parsed_opts.minor);

	int i = 0;
	while(1)
	{
		int status = prompt_for_write(dev);
		if(status < 0) fprint_status( stderr, "ibdev error\n");
	
		perform_read(dev);
		if(status < 0) fprint_status( stderr, "ibdev error\n");
		i++;
		if(i%1000 == 0) printf("%d\n",i);
	}

	ibonl(dev, 0);
	return 0;
}

/*
* This is a simple error handling function
*
*/

void fprint_status( FILE* filep, char *msg )
{
	fprintf( filep, "%s\n", msg);

	fprintf( filep, "ibsta = 0x%x  < ", ThreadIbsta() );
	if(ThreadIbsta() & ERR)  fprintf(filep, "ERR ");
	if(ThreadIbsta() & TIMO) fprintf(filep, "TIMO ");
	if(ThreadIbsta() & END)  fprintf(filep, "END ");
	if(ThreadIbsta() & SRQI) fprintf(filep, "SRQI ");
	if(ThreadIbsta() & RQS) fprintf(filep, "RQS ");
	if(ThreadIbsta() & SPOLL) fprintf(filep, "SPOLL ");
	if(ThreadIbsta() & EVENT) fprintf(filep, "EVENT ");
	if(ThreadIbsta() & CMPL) fprintf(filep, "CMPL ");
	if(ThreadIbsta() & LOK) fprintf(filep, "LOK ");
	if(ThreadIbsta() & REM)  fprintf(filep, "REM ");
	if(ThreadIbsta() & CIC)  fprintf(filep, "CIC ");
	if(ThreadIbsta() & ATN)  fprintf(filep, "ATN ");
	if(ThreadIbsta() & TACS) fprintf(filep, "TACS ");
	if(ThreadIbsta() & LACS) fprintf(filep, "LACS ");
	if(ThreadIbsta() & DCAS) fprintf(filep, "DCAS ");
	if(ThreadIbsta() & DTAS) fprintf(filep, "DTAS ");
	fprintf( filep, ">\n" );

	fprintf( filep,"iberr= %d\n", iberr);
	if( ( ThreadIbsta() & ERR ) )
	{
		fprintf( filep, "%s\n", gpib_error_string( ThreadIberr() ) );
	}

	fprintf( filep, "\n" );

	fprintf( filep, "ibcnt = %d\n", ibcnt );
}

